@@ -1,8 +1,11 @@
-# Copyright (c) 2016 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Copyright (c) 2018 Ultimaker B.V.
+# Uranium is released under the terms of the LGPLv3 or higher.
+
+from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING
 
 from UM.PluginRegistry import PluginRegistry
 from UM.Logger import Logger
+from .FileReader import FileReader
 from .FileWriter import FileWriter
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSlot, QUrl
 
@@ -11,46 +14,47 @@ from UM.Platform import Platform
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
-MYPY = False
-if MYPY:
-    from UM.Application import Application
+if TYPE_CHECKING:
+    from UM.Qt.QtApplication import QtApplication
+    from UM.Mesh.MeshReader import MeshReader
+    from UM.Mesh.MeshWriter import MeshWriter
 
 ##  Central class for reading and writing meshes.
 #   This class is created by Application and handles reading and writing mesh files.
 class FileHandler(QObject):
     _instance = None    # type: FileHandler
-    _application = None # type: Application
+    _application = None # type: QtApplication
 
-    def __init__(self, writer_type, reader_type, parent = None):
+    def __init__(self, writer_type: str = "unknown_file_writer", reader_type: str = "unknown_file_reader", parent: QObject = None) -> None:
         super().__init__(parent)
 
-        self._readers = {}
-        self._writers = {}
+        self._readers = {} # type: Dict[str, MeshReader]
+        self._writers = {} # type: Dict[str, MeshWriter]
 
-        self._writer_type = writer_type
-        self._reader_type = reader_type
+        self._writer_type = writer_type # type: str
+        self._reader_type = reader_type # type: str
 
         PluginRegistry.addType(self._writer_type, self.addWriter)
         PluginRegistry.addType(self._reader_type, self.addReader)
 
     @classmethod
-    def setApplication(cls, application):
+    def setApplication(cls, application: "QtApplication") -> None:
         cls._application = application
 
     @classmethod
-    def getApplication(cls):
+    def getApplication(cls) -> "QtApplication":
         return cls._application
 
-    ##  Return the singleton instance of the filehandler.
+    ##  Return the singleton instance of the file handler.
     @classmethod
-    def getInstance(cls, *args, **kwargs) -> "FileHandler":
+    def getInstance(cls, *args: Any, **kwargs: Any) -> "FileHandler":
         if not cls._instance:
             cls._instance = cls()
 
         return cls._instance
 
-    @pyqtProperty("QStringList", constant=True)
-    def supportedReadFileTypes(self):
+    @pyqtProperty("QStringList", constant = True)
+    def supportedReadFileTypes(self) -> List[str]:
         file_types = []
         all_types = []
 
@@ -69,8 +73,8 @@ class FileHandler(QObject):
 
         return file_types
 
-    @pyqtProperty("QStringList", constant=True)
-    def supportedWriteFileTypes(self):
+    @pyqtProperty("QStringList", constant = True)
+    def supportedWriteFileTypes(self) -> List[str]:
         file_types = []
 
         for item in self.getSupportedFileTypesWrite():
@@ -81,17 +85,17 @@ class FileHandler(QObject):
         return file_types
 
     @pyqtSlot(QUrl)
-    def readLocalFile(self, file):
+    def readLocalFile(self, file: QUrl) -> None:
         if not file.isValid():
             return
         self._readLocalFile(file)
 
-    def _readLocalFile(self, file):
+    def _readLocalFile(self, file: QUrl) -> None:
         raise NotImplementedError("_readLocalFile needs to be implemented by subclasses")
 
     ##  Get list of all supported filetypes for writing.
     #   \return List of dicts containing id, extension, description and mime_type for all supported file types.
-    def getSupportedFileTypesWrite(self):
+    def getSupportedFileTypesWrite(self) -> List[Dict[str, Union[str, int]]]:
         supported_types = []
         meta_data = PluginRegistry.getInstance().getAllMetaData(filter={self._writer_type: {}}, active_only=True)
         for entry in meta_data:
@@ -110,8 +114,8 @@ class FileHandler(QObject):
         return supported_types
 
     # Get list of all supported file types for reading.
-    # \returns List of strings with all supported file types.
-    def getSupportedFileTypesRead(self):
+    # \returns For each supported file type, the description of the plug-in.
+    def getSupportedFileTypesRead(self) -> Dict[str, str]:
         supported_types = {}
         meta_data = PluginRegistry.getInstance().getAllMetaData(filter={self._reader_type: {}}, active_only=True)
         for entry in meta_data:
@@ -123,10 +127,10 @@ class FileHandler(QObject):
                         supported_types[ext] = description
         return supported_types
 
-    def addReader(self, reader):
+    def addReader(self, reader: "MeshReader") -> None:
         self._readers[reader.getPluginId()] = reader
 
-    def addWriter(self, writer):
+    def addWriter(self, writer: "MeshWriter") -> None:
         self._writers[writer.getPluginId()] = writer
 
     # Try to read the data from a file using a specified Reader.
@@ -134,7 +138,7 @@ class FileHandler(QObject):
     # \param file_name The name of the file to load.
     # \param kwargs Keyword arguments.
     # \returns None if nothing was found
-    def readerRead(self, reader, file_name, **kwargs):
+    def readerRead(self, reader: FileReader, file_name: str, **kwargs: Any):
         raise NotImplementedError("readerRead must be implemented by subclasses.")
 
     ##  Get a mesh writer object that supports writing the specified mime type
@@ -142,7 +146,7 @@ class FileHandler(QObject):
     #   \param mime The mime type that should be supported.
     #   \return A MeshWriter instance or None if no mesh writer supports the specified mime type. If there are multiple
     #           writers that support the specified mime type, the first entry is returned.
-    def getWriterByMimeType(self, mime):
+    def getWriterByMimeType(self, mime: str) -> Optional["MeshWriter"]:
         writer_data = PluginRegistry.getInstance().getAllMetaData(filter={self._writer_type: {}}, active_only=True)
         for entry in writer_data:
             for output in entry[self._writer_type].get("output", []):
@@ -152,7 +156,7 @@ class FileHandler(QObject):
         return None
 
     ##  Get an instance of a mesh writer by ID
-    def getWriter(self, writer_id):
+    def getWriter(self, writer_id: str) -> Optional["MeshWriter"]:
         if writer_id not in self._writers:
             return None
 
@@ -161,7 +165,7 @@ class FileHandler(QObject):
     ##  Find a Reader that accepts the given file name.
     #   \param file_name The name of file to load.
     #   \returns Reader that accepts the given file name. If no acceptable Reader is found None is returned.
-    def getReaderForFile(self, file_name):
+    def getReaderForFile(self, file_name: str) -> Optional["MeshReader"]:
         for id, reader in self._readers.items():
             try:
                 if reader.acceptsFile(file_name):
