@@ -1,11 +1,11 @@
-# Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Copyright (c) 2018 Ultimaker B.V.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 import time
-
-from UM.Signal import Signal, signalemitter
+from typing import Any
 
 from UM.JobQueue import JobQueue
+from UM.Signal import Signal, signalemitter
 
 
 ##  Base class for things that should be performed in a thread.
@@ -20,7 +20,7 @@ class Job:
         super().__init__()
         self._running = False   # type: bool
         self._finished = False  # type: bool
-        self._result = None     # type: any
+        self._result = None     # type: Any
         self._error = None
 
     ##  Perform the actual task of this job. Should be reimplemented by subclasses.
@@ -31,13 +31,13 @@ class Job:
     ##  Get the result of the job.
     #
     #   The actual result object returned by this method is dependant on the implementation.
-    def getResult(self):
+    def getResult(self) -> Any:
         return self._result
 
     ##  Set the result of this job.
     #
     #   This should be called by run() to set the actual result of the Job.
-    def setResult(self, result: any):
+    def setResult(self, result: Any):
         self._result = result
 
     ##  Set an exception that was thrown while the job was being executed.
@@ -46,7 +46,7 @@ class Job:
     #   to execute properly.
     #
     #   \param error \type{Exception} The exception to set.
-    def setError(self, error: Exception):
+    def setError(self, error: Exception) -> None:
         self._error = error
 
     ##  Start the job.
@@ -54,14 +54,14 @@ class Job:
     #   This will put the Job into the JobQueue to be processed whenever a thread is available.
     #
     #   \sa JobQueue::add()
-    def start(self):
+    def start(self) -> None:
         JobQueue.getInstance().add(self)
 
     ##  Cancel the job.
     #
     #   This will remove the Job from the JobQueue. If the run() function has already been called,
     #   this will do nothing.
-    def cancel(self):
+    def cancel(self) -> None:
         JobQueue.getInstance().remove(self)
 
     ##  Check whether the job is currently running.
@@ -105,5 +105,5 @@ class Job:
     #   forces a GIL release and allows a different thread to start processing
     #   if it is waiting.
     @staticmethod
-    def yieldThread():
+    def yieldThread() -> None:
         time.sleep(0)  # Sleeping for 0 introduces no delay but does allow context switching.
