@@ -12,6 +12,11 @@ from UM.Application import Application
 from UM.Preferences import Preferences
 from UM.Signal import Signal, signalemitter
 
+from typing import Optional
+
+MYPY = False
+if MYPY:
+    from PyQt5.QtQuick import QQuickItem
 
 ##  QQuickWindow subclass that provides the main window.
 @signalemitter
@@ -28,9 +33,14 @@ class MainWindow(QQuickWindow):
         self._mouse_device.setPluginId("qt_mouse")
         self._key_device = QtKeyDevice()
         self._key_device.setPluginId("qt_key")
-        self._previous_focus = None
+        self._previous_focus = None  # type: Optional["QQuickItem"]
 
         self._app = QCoreApplication.instance()
+
+        # Remove previously added input devices (if any). This can happen if the window was re-loaded.
+        self._app.getController().removeInputDevice("qt_mouse")
+        self._app.getController().removeInputDevice("qt_key")
+
         self._app.getController().addInputDevice(self._mouse_device)
         self._app.getController().addInputDevice(self._key_device)
         self._app.getController().getScene().sceneChanged.connect(self._onSceneChanged)
@@ -60,11 +70,29 @@ class MainWindow(QQuickWindow):
         self._mouse_x = 0
         self._mouse_y = 0
 
+        self._mouse_pressed = False
+
         self._viewport_rect = QRectF(0, 0, 1.0, 1.0)
 
         Application.getInstance().setMainWindow(self)
         self._fullscreen = False
 
+        self._allow_resize = True
+
+    def setAllowResize(self, allow_resize: bool):
+        if self._allow_resize != allow_resize:
+            if not allow_resize:
+                self.setMaximumHeight(self.height())
+                self.setMinimumHeight(self.height())
+                self.setMaximumWidth(self.width())
+                self.setMinimumWidth(self.width())
+            else:
+                self.setMaximumHeight(16777215)
+                self.setMinimumHeight(0)
+                self.setMaximumWidth(16777215)
+                self.setMinimumWidth(0)
+            self._allow_resize = allow_resize
+
     @pyqtSlot()
     def toggleFullscreen(self):
         if self._fullscreen:
@@ -118,11 +146,14 @@ class MainWindow(QQuickWindow):
 
         self._previous_focus = self.activeFocusItem()
         self._mouse_device.handleEvent(event)
+        self._mouse_pressed = True
 
     def mouseMoveEvent(self, event):
         self._mouse_x = event.x()
         self._mouse_y = event.y()
-        self.mousePositionChanged.emit()
+
+        if self._mouse_pressed and self._app.getController().isModelRenderingEnabled():
+            self.mousePositionChanged.emit()
 
         super().mouseMoveEvent(event)
         if event.isAccepted():
@@ -135,6 +166,7 @@ class MainWindow(QQuickWindow):
         if event.isAccepted():
             return
         self._mouse_device.handleEvent(event)
+        self._mouse_pressed = False
 
     def keyPressEvent(self, event):
         super().keyPressEvent(event)
@@ -171,7 +203,8 @@ class MainWindow(QQuickWindow):
         QMetaObject.invokeMethod(self, "_onWindowGeometryChanged", Qt.QueuedConnection)
 
     def hideEvent(self, event):
-        Application.getInstance().windowClosed()
+        if Application.getInstance().getMainWindow() == self:
+            Application.getInstance().windowClosed()
 
     renderCompleted = Signal(type = Signal.Queued)
 
@@ -200,20 +233,22 @@ class MainWindow(QQuickWindow):
         elif self.windowState() == Qt.WindowMaximized:
             self._preferences.setValue("general/window_state", Qt.WindowMaximized)
 
-    def _updateViewportGeometry(self, width, height):
+    def _updateViewportGeometry(self, width: int, height: int):
         view_width = width * self._viewport_rect.width()
         view_height = height * self._viewport_rect.height()
 
         for camera in self._app.getController().getScene().getAllCameras():
-            camera.setViewportSize(view_width, view_height)
             camera.setWindowSize(width, height)
-            projection_matrix = Matrix()
-            if camera.isPerspective():
-                if view_width is not 0:
-                    projection_matrix.setPerspective(30, view_width / view_height, 1, 500)
-            else:
-                projection_matrix.setOrtho(-view_width / 2, view_width / 2, -view_height / 2, view_height / 2, -500, 500)
-            camera.setProjectionMatrix(projection_matrix)
+
+            if camera.getAutoAdjustViewPort():
+                camera.setViewportSize(view_width, view_height)
+                projection_matrix = Matrix()
+                if camera.isPerspective():
+                    if view_width is not 0:
+                        projection_matrix.setPerspective(30, view_width / view_height, 1, 500)
+                else:
+                    projection_matrix.setOrtho(-view_width / 2, view_width / 2, -view_height / 2, view_height / 2, -500, 500)
+                camera.setProjectionMatrix(projection_matrix)
 
         self._app.getRenderer().setViewportSize(view_width, view_height)
         self._app.getRenderer().setWindowSize(width, height)
