@@ -1,5 +1,5 @@
 # Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 from UM.Qt.ListModel import ListModel
 from UM.Application import Application
@@ -15,8 +15,10 @@ class VisibleMessagesModel(ListModel):
     IconRole = Qt.UserRole + 6
     DescriptionRole = Qt.UserRole + 7
     DismissableRole = Qt.UserRole + 8
-    
-    def __init__(self, parent = None):
+    TileRole = Qt.UserRole + 9
+    StyleRole = Qt.UserRole + 10
+
+    def __init__(self, parent=None):
         super().__init__(parent)
         Application.getInstance().visibleMessageAdded.connect(self.addMessage)
         Application.getInstance().visibleMessageRemoved.connect(self.removeMessage)
@@ -26,21 +28,23 @@ class VisibleMessagesModel(ListModel):
         self.addRoleName(self.IDRole, "id")
         self.addRoleName(self.ActionsRole, "actions")
         self.addRoleName(self.DismissableRole, "dismissable")
+        self.addRoleName(self.TileRole, "title")
         self._populateMessageList()
-    
+
     def _populateMessageList(self):
         for message in Application.getInstance().getVisibleMessages():
             self.addMessage(message)
-    
+
     def addMessage(self, message):
         self.appendItem({
-                "text": message.getText(),
-                "progress": message.getProgress(),
-                "max_progress": message.getMaxProgress(),
-                "id": str(id(message)),
-                "actions":self.createActionsModel(message.getActions()),
-                "dismissable": message.isDismissable()
-            })
+            "text": message.getText(),
+            "progress": message.getProgress(),
+            "max_progress": message.getMaxProgress(),
+            "id": str(id(message)),
+            "actions": self.createActionsModel(message.getActions()),
+            "dismissable": message.isDismissable(),
+            "title": message.getTitle()
+        })
         message.progressChanged.connect(self._onMessageProgress)
 
     def createActionsModel(self, actions):
@@ -49,22 +53,23 @@ class VisibleMessagesModel(ListModel):
         model.addRoleName(self.TextRole,"name")
         model.addRoleName(self.IconRole, "icon")
         model.addRoleName(self.DescriptionRole, "description")
-        
+        model.addRoleName(self.StyleRole, "button_style")
+
         for action in actions:
             model.appendItem(action)
-        return model   
-    
+        return model
+
     @pyqtSlot(str)
     def hideMessage(self, message_id):
         Application.getInstance().hideMessageById(message_id)
-    
+
     @pyqtSlot(str, str)
     def actionTriggered(self, message_id, action_id):
         for message in Application.getInstance().getVisibleMessages():
             if str(id(message)) == message_id:
                 message.actionTriggered.emit(message, action_id)
                 break
-    
+
     def removeMessage(self, message):
         message_id = str(id(message))
         for index in range(0,len(self.items)):
