@@ -1,4 +1,4 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import collections
@@ -249,7 +249,7 @@ class SettingDefinitionsModel(QAbstractListModel):
 
     ##  Show the children of a specified SettingDefinition and all children of those settings as well.
     @pyqtSlot(str)
-    def expandAll(self, key):
+    def expandRecursive(self, key):
         if not self._container:
             return
 
@@ -260,7 +260,7 @@ class SettingDefinitionsModel(QAbstractListModel):
 
         for child in definitions[0].children:
             if child.children:
-                self.expandAll(child.key)
+                self.expandRecursive(child.key)
 
     ##  Hide the children of a specified SettingDefinition.
     @pyqtSlot(str)
@@ -295,7 +295,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         self.setVisible(key, False)
 
     @pyqtSlot(bool)
-    def setAllVisible(self, visible):
+    def setAllExpandedVisible(self, visible):
         new_visible = set()
 
         for index in self._row_index_list:
@@ -308,6 +308,19 @@ class SettingDefinitionsModel(QAbstractListModel):
         else:
             self._visibility_handler.setVisible(self._visible - new_visible)
 
+    @pyqtSlot(bool)
+    def setAllVisible(self, visible):
+        new_visible = set()
+
+        for definition in self._definition_list:
+            if definition.type != "category":
+                new_visible.add(definition.key)
+
+        if visible:
+            self._visibility_handler.setVisible(new_visible | self._visible)
+        else:
+            self._visibility_handler.setVisible(self._visible - new_visible)
+
     ##  Set a single SettingDefinition's visible state
     @pyqtSlot(str, bool)
     def setVisible(self, key, visible):
