@@ -1,17 +1,17 @@
-# Copyright (c) 2016 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Copyright (c) 2018 Ultimaker B.V.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 import ast
+import builtins #To check against functions that are built-in in Python.
 import math # Imported here so it can be used easily by the setting functions.
-from typing import Any, Dict, Callable, Set, FrozenSet, NamedTuple, Optional
+from typing import Any, Callable, Dict, FrozenSet, NamedTuple, Optional, Set, TYPE_CHECKING
 
 from UM.Settings.Interfaces import ContainerInterface
 from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext
 from UM.Logger import Logger
 
-MYPY = False
-if MYPY:
-    from UM.Settings.SettingInstance import SettingInstance
+if TYPE_CHECKING:
+    from typing import FrozenSet
 
 class IllegalMethodError(Exception):
     pass
@@ -26,17 +26,17 @@ class SettingFunction:
     ##  Constructor.
     #
     #   \param code The Python code this function should evaluate.
-    def __init__(self, code: str) -> None:
+    def __init__(self, expression: str) -> None:
         super().__init__()
 
-        self._code = code
+        self._code = expression
 
         #  Keys of all settings that are referenced to in this function.
-        self._used_keys = frozenset()  # type: frozenset[str]
-        self._used_values = frozenset()
+        self._used_keys = frozenset()  # type: FrozenSet[str]
+        self._used_values = frozenset() # type: FrozenSet[str]
 
-        self._compiled = None
-        self._valid = False  # type: str
+        self._compiled = None #type: ignore #Actually an Optional['code'] object, but Python doesn't properly expose this 'code' object via any library.
+        self._valid = False  # type: bool
 
         try:
             tree = ast.parse(self._code, "eval")
@@ -55,6 +55,9 @@ class SettingFunction:
             Logger.log("e", "Exception in function ({0}) for setting: {1}".format(str(e), self._code))
 
     ##  Call the actual function to calculate the value.
+    #
+    #   \param value_provider The container from which to get setting values in
+    #   the formula.
     def __call__(self, value_provider: ContainerInterface, context: Optional[PropertyEvaluationContext] = None) -> Any:
         if not value_provider:
             return None
@@ -76,6 +79,9 @@ class SettingFunction:
         g = {}  # type: Dict[str, Any]
         g.update(globals())
         g.update(self.__operators)
+        # override operators if there is any in the context
+        if context is not None:
+            g.update(context.context.get("override_operators", {}))
 
         try:
             return eval(self._compiled, g, locals)
@@ -83,7 +89,7 @@ class SettingFunction:
             Logger.logException("d", "An exception occurred in inherit function %s", self)
             return 0  # Settings may be used in calculations and they need a value
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: object) -> bool:
         if not isinstance(other, SettingFunction):
             return False
 
@@ -143,26 +149,26 @@ _VisitResult = NamedTuple("_VisitResult", [("values", Set[str]), ("keys", Set[st
 # setting keys (strings) that are used by the expression, whereas "used values" are
 # actual variable references that are needed for the function to be executed.
 class _SettingExpressionVisitor(ast.NodeVisitor):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
-        self.values = set()
-        self.keys = set()
+        self.values = set() #type: Set[str]
+        self.keys = set() #type: Set[str]
 
     def visit(self, node: ast.AST) -> _VisitResult:
         super().visit(node)
         return _VisitResult(values = self.values, keys = self.keys)
 
-    def visit_Name(self, node: ast.AST) -> None: # [CodeStyle: ast.NodeVisitor requires this function name]
+    def visit_Name(self, node: ast.Name) -> None: # [CodeStyle: ast.NodeVisitor requires this function name]
         if node.id in self._blacklist:
             raise IllegalMethodError(node.id)
 
-        if node.id not in self._knownNames and node.id not in __builtins__:
+        if node.id not in self._knownNames and node.id not in dir(builtins):
             self.values.add(node.id)
             self.keys.add(node.id)
 
     def visit_Str(self, node: ast.AST) -> None:
-        if node.s not in self._knownNames and node.s not in __builtins__:
-            self.keys.add(node.s)
+        if node.s not in self._knownNames and node.s not in dir(builtins): #type: ignore #AST uses getattr stuff, so ignore type of node.s.
+            self.keys.add(node.s) #type: ignore
 
     _knownNames = {
         "math",
@@ -171,7 +177,7 @@ class _SettingExpressionVisitor(ast.NodeVisitor):
         "debug",
         "sum",
         "len"
-    }
+    } #type: Set[str]
 
     _blacklist = {
         "sys",
@@ -181,4 +187,4 @@ class _SettingExpressionVisitor(ast.NodeVisitor):
         "eval",
         "exec",
         "subprocess",
-    }
+    } #type: Set[str]
