@@ -54,7 +54,7 @@ class MirrorTool(Tool):
                 self.operationStopped.emit(self)
 
             # Perform a mirror operation
-            if self.getLockedAxis():
+            if self.getLockedAxis() != ToolHandle.NoAxis:
                 op = None
                 if Selection.getCount() == 1:
                     node = Selection.getSelectedObject(0)
@@ -70,7 +70,7 @@ class MirrorTool(Tool):
                 else:
                     op = GroupedOperation()
 
-                    for node in Selection.getAllSelectedObjects():
+                    for node in self._getSelectedObjectsWithoutSelectedAncestors():
                         if self.getLockedAxis() == ToolHandle.XAxis:
                             mirror = Vector(-1, 1, 1)
                         elif self.getLockedAxis() == ToolHandle.YAxis:
@@ -84,7 +84,7 @@ class MirrorTool(Tool):
 
                 op.push()
 
-                self.setLockedAxis(None)
+                self.setLockedAxis(ToolHandle.NoAxis)
                 return True
 
         return False
