@@ -1,6 +1,6 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-
+from UM.Benchmark import Benchmark
 from UM.Tool import Tool
 from UM.Event import Event, MouseEvent, KeyEvent
 from UM.Scene.ToolHandle import ToolHandle
@@ -66,11 +66,11 @@ class ScaleTool(Tool):
         super().event(event)
 
         if event.type == Event.ToolActivateEvent:
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 node.boundingBoxChanged.connect(self.propertyChanged)
 
         if event.type == Event.ToolDeactivateEvent:
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 node.boundingBoxChanged.disconnect(self.propertyChanged)
 
         # Handle modifier keys: Shift toggles snap, Control toggles uniform scaling
@@ -105,7 +105,7 @@ class ScaleTool(Tool):
 
             # Save the current positions of the node, as we want to scale arround their current centres
             self._saved_node_positions = []
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 self._saved_node_positions.append((node, node.getPosition()))
 
             self._scale_sum = 0.0
@@ -181,7 +181,7 @@ class ScaleTool(Tool):
             # Finish a scale operation
             if self.getDragPlane():
                 self.setDragPlane(None)
-                self.setLockedAxis(None)
+                self.setLockedAxis(ToolHandle.NoAxis)
                 self._drag_length = 0
                 self.operationStopped.emit(self)
                 return True
@@ -256,7 +256,7 @@ class ScaleTool(Tool):
     def getScaleX(self):
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).x), 4))
+            return abs(round(float(Selection.getSelectedObject(0).getScale().x), 4))
 
         return 1.0
 
@@ -266,7 +266,7 @@ class ScaleTool(Tool):
     def getScaleY(self):
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).y), 4))
+            return abs(round(float(Selection.getSelectedObject(0).getScale().y), 4))
 
         return 1.0
 
@@ -276,7 +276,7 @@ class ScaleTool(Tool):
     def getScaleZ(self):
         if Selection.hasSelection():
             ## Ensure that the returned value is positive (mirror causes scale to be negative)
-            return abs(round(float(self._getScaleInWorldCoordinates(Selection.getSelectedObject(0)).z), 4))
+            return abs(round(float(Selection.getSelectedObject(0).getScale().z), 4))
 
         return 1.0
 
@@ -296,7 +296,7 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
@@ -317,7 +317,7 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
@@ -338,7 +338,7 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
@@ -349,7 +349,7 @@ class ScaleTool(Tool):
     def setScaleX(self, scale):
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = self._getScaleInWorldCoordinates(obj)
+            obj_scale = obj.getScale()
             if round(float(obj_scale.x), 4) != scale:
                 scale_factor = abs(scale / obj_scale.x)
                 if self._non_uniform_scale:
@@ -358,7 +358,7 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
@@ -369,7 +369,7 @@ class ScaleTool(Tool):
     def setScaleY(self, scale):
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = self._getScaleInWorldCoordinates(obj)
+            obj_scale = obj.getScale()
             if round(float(obj_scale.y), 4) != scale:
                 scale_factor = abs(scale / obj_scale.y)
                 if self._non_uniform_scale:
@@ -378,7 +378,7 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
@@ -389,7 +389,7 @@ class ScaleTool(Tool):
     def setScaleZ(self, scale):
         obj = Selection.getSelectedObject(0)
         if obj:
-            obj_scale = self._getScaleInWorldCoordinates(obj)
+            obj_scale = obj.getScale()
             if round(float(obj_scale.z), 4) != scale:
                 scale_factor = abs(scale / obj_scale.z)
                 if self._non_uniform_scale:
@@ -398,12 +398,13 @@ class ScaleTool(Tool):
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
                 op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
+                for node in self._getSelectedObjectsWithoutSelectedAncestors():
                     op.addOperation(
                         ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
                 op.push()
 
     ##  Convenience function that gives the scale of an object in the coordinate space of the world.
+    #   The function might return wrong value if the grouped models are rotated
     #
     #   \param node type(SceneNode)
     #   \return scale type(float) scale factor (1.0 = normal scale)
